'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mergeModules = mergeModules;
exports.loadModules = loadModules;

require('core-js/modules/es6.typed.array-buffer');

require('core-js/modules/es6.typed.int8-array');

require('core-js/modules/es6.typed.uint8-array');

require('core-js/modules/es6.typed.uint8-clamped-array');

require('core-js/modules/es6.typed.int16-array');

require('core-js/modules/es6.typed.uint16-array');

require('core-js/modules/es6.typed.int32-array');

require('core-js/modules/es6.typed.uint32-array');

require('core-js/modules/es6.typed.float32-array');

require('core-js/modules/es6.typed.float64-array');

require('core-js/modules/es6.map');

require('core-js/modules/es6.set');

require('core-js/modules/es6.weak-map');

require('core-js/modules/es6.weak-set');

require('core-js/modules/es6.reflect.apply');

require('core-js/modules/es6.reflect.construct');

require('core-js/modules/es6.reflect.define-property');

require('core-js/modules/es6.reflect.delete-property');

require('core-js/modules/es6.reflect.get');

require('core-js/modules/es6.reflect.get-own-property-descriptor');

require('core-js/modules/es6.reflect.get-prototype-of');

require('core-js/modules/es6.reflect.has');

require('core-js/modules/es6.reflect.is-extensible');

require('core-js/modules/es6.reflect.own-keys');

require('core-js/modules/es6.reflect.prevent-extensions');

require('core-js/modules/es6.reflect.set');

require('core-js/modules/es6.reflect.set-prototype-of');

require('core-js/modules/es6.promise');

require('core-js/modules/es6.symbol');

require('core-js/modules/es6.function.name');

require('core-js/modules/es6.regexp.flags');

require('core-js/modules/es6.regexp.match');

require('core-js/modules/es6.regexp.replace');

require('core-js/modules/es6.regexp.split');

require('core-js/modules/es6.regexp.search');

require('core-js/modules/es6.array.from');

require('core-js/modules/es7.array.includes');

require('core-js/modules/es7.object.values');

require('core-js/modules/es7.object.entries');

require('core-js/modules/es7.object.get-own-property-descriptors');

require('core-js/modules/es7.string.pad-start');

require('core-js/modules/es7.string.pad-end');

require('regenerator-runtime/runtime');

var _graphqlTag = require('graphql-tag');

var _graphqlTag2 = _interopRequireDefault(_graphqlTag);

var _printer = require('graphql/language/printer');

var _deepExtend = require('deep-extend');

var _deepExtend2 = _interopRequireDefault(_deepExtend);

var _requireAll = require('require-all');

var _requireAll2 = _interopRequireDefault(_requireAll);

var _values = require('core-js/fn/object/values');

var _values2 = _interopRequireDefault(_values);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function mergeModules() {
  var modules = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

  var typeDefs = mergeTypeDefs(modules.filter(function (_ref) {
    var typeDefs = _ref.typeDefs;
    return !!typeDefs;
  }).map(function (_ref2) {
    var typeDefs = _ref2.typeDefs;
    return typeDefs;
  }));
  var resolvers = mergeResolvers(modules.filter(function (_ref3) {
    var resolvers = _ref3.resolvers;
    return !!resolvers;
  }).map(function (_ref4) {
    var resolvers = _ref4.resolvers;
    return resolvers;
  }));
  return {
    typeDefs: [typeDefs],
    resolvers
  };
};

function loadModules(directory) {
  var modules = [];
  (0, _requireAll2.default)({
    dirname: directory,
    recursive: true,
    resolve(mod) {
      modules.push(mod);
    }
  });
  return mergeModules(modules);
};

var typeDefsToStr = function typeDefsToStr() {
  var typeDefs = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

  if (typeof typeDefs === 'string') {
    return typeDefs;
  } else if (typeof typeDefs === 'function') {
    return typeDefsToStr(typeDefs());
  } else {
    return typeDefs.map(function (typeDef) {
      return typeDefsToStr(typeDef);
    }).join('\n');
  }
};

var mergeTypeDefs = function mergeTypeDefs() {
  var typeDefs = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

  typeDefs = typeDefs.map(function (typeDefs) {
    return _graphqlTag2.default`${typeDefsToStr(typeDefs)}`;
  });
  var rootTypes = {};
  typeDefs.forEach(function (graphql) {
    graphql.definitions.forEach(function (def) {
      var name = def.name.value;
      if (rootTypes[name]) {
        // throw error if enum with same name is defined twice
        if (rootTypes[name].kind === 'EnumTypeDefinition') {
          throw new Error(`duplicate enum definition '${rootTypes[name].name.value}'`);
        }
        rootTypes[name].fields = rootTypes[name].fields.concat(def.fields);
      } else {
        rootTypes[name] = def;
      }
    });
  });
  return (0, _printer.print)({
    kind: 'Document',
    definitions: Object.values(rootTypes)
  });
};

var mergeResolvers = function mergeResolvers() {
  var resolvers = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

  var rootResolvers = {};
  resolvers.forEach(function (_resolvers) {
    (0, _deepExtend2.default)(rootResolvers, _resolvers);
  });
  return rootResolvers;
};